@@ -1,4 +1,4 @@
-# Copyright 2008-2011 Kouhei Sutou <kou@cozmixng.org>
+# Copyright 2008-2018 Kouhei Sutou <kou@cozmixng.org>
 #
 # This library is free software; you can redistribute it and/or
 # modify it under the terms of the GNU Lesser General Public
@@ -12,11 +12,15 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this library; if not, write to the Free Software
-# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+begin
+  require "pkg-config/version"
+rescue LoadError
+end
 
 require "rbconfig"
 
-require 'mkmf'
 require 'shellwords'
 require 'English'
 require 'pathname'
@@ -41,6 +45,14 @@ class PackageConfig
     end
 
     private
+    def with_config(config, default=nil)
+      if defined?(super)
+        super
+      else
+        default
+      end
+    end
+
     def guess_native_pkg_config
       pkg_config = with_config("pkg-config", ENV["PKG_CONFIG"] || "pkg-config")
       pkg_config = Pathname.new(pkg_config)
@@ -85,10 +97,14 @@ class PackageConfig
         rescue DL::DLError
           return nil
         end
-        extern "const char *dln_find_exe(const char *, const char *)"
+        begin
+          extern "const char *dln_find_exe(const char *, const char *)"
+        rescue DL::DLError
+          return nil
+        end
       end
       path = dln.dln_find_exe(pkg_config.to_s, nil)
-      if path.size.zero?
+      if path.nil? or path.size.zero?
         nil
       else
         Pathname(path.to_s)
@@ -134,6 +150,10 @@ class PackageConfig
     collect_cflags[0].join(" ")
   end
 
+  def cflags_only_other
+    collect_cflags[1].join(" ")
+  end
+
   def libs
     path_flags, other_flags = collect_libs
     (path_flags + other_flags).join(" ")
@@ -187,12 +207,17 @@ class PackageConfig
 
   private
   def collect_cflags
-    all_cflags = all_required_packages.collect do |package|
+    cflags_set = [declaration("Cflags")]
+    cflags_set += required_packages.collect do |package|
       self.class.new(package, @options).cflags
     end
-    all_cflags = [declaration("Cflags")] + all_cflags
-    all_cflags = all_cflags.join(" ").gsub(/-I /, '-I').split.uniq
+    cflags_set += private_required_packages.collect do |package|
+      self.class.new(package, @options).cflags
+    end
+    all_cflags = normalize_cflags(Shellwords.split(cflags_set.join(" ")))
     path_flags, other_flags = all_cflags.partition {|flag| /\A-I/ =~ flag}
+    path_flags = normalize_path_flags(path_flags, "-I")
+    path_flags = remove_duplicated_include_paths(path_flags)
     path_flags = path_flags.reject do |flag|
       flag == "-I/usr/include"
     end
@@ -204,6 +229,45 @@ class PackageConfig
     [path_flags, other_flags]
   end
 
+  def normalize_path_flags(path_flags, flag_option)
+    path_flags.collect do |path_flag|
+      path = path_flag.sub(flag_option, "")
+      prefix = ""
+      case RUBY_PLATFORM
+      when "x86-mingw32"
+        prefix = Dir.glob("c:/msys{32,64,*}").first
+      when "x64-mingw32"
+        prefix = Dir.glob("c:/msys{64,*}").first
+      end
+      if /\A[a-z]:/i === path
+        "#{flag_option}#{path}"
+      else
+        "#{flag_option}#{prefix}#{path}"
+      end
+    end
+  end
+
+  def normalize_cflags(cflags)
+    normalized_cflags = []
+    enumerator = cflags.to_enum
+    begin
+      loop do
+        cflag = enumerator.next
+        normalized_cflags << cflag
+        case cflag
+        when "-I"
+          normalized_cflags << enumerator.next
+        end
+      end
+    rescue StopIteration
+    end
+    normalized_cflags
+  end
+
+  def remove_duplicated_include_paths(path_flags)
+    path_flags.uniq
+  end
+
   def collect_libs
     all_libs = required_packages.collect do |package|
       self.class.new(package, @options).libs
@@ -211,8 +275,9 @@ class PackageConfig
     all_libs = [declaration("Libs")] + all_libs
     all_libs = all_libs.join(" ").gsub(/-([Ll]) /, '\1').split.uniq
     path_flags, other_flags = all_libs.partition {|flag| /\A-L/ =~ flag}
+    path_flags = normalize_path_flags(path_flags, "-L")
     path_flags = path_flags.reject do |flag|
-      /\A-L\/usr\/lib(?:64)?\z/ =~ flag
+      /\A-L\/usr\/lib(?:64|x32)?\z/ =~ flag
     end
     if @msvc_syntax
       path_flags = path_flags.collect do |flag|
@@ -271,29 +336,66 @@ class PackageConfig
   end
 
   def guess_default_path
-    arch_depended_path = Dir.glob('/usr/lib/*/pkgconfig').join(SEPARATOR)
-    default_path = ["/usr/local/lib64/pkgconfig",
-                    "/usr/local/lib/pkgconfig",
-                    "/usr/local/libdata/pkgconfig",
-                    "/opt/local/lib/pkgconfig",
-                    arch_depended_path,
-                    "/usr/lib64/pkgconfig",
-                    "/usr/lib/pkgconfig",
-                    "/usr/X11/lib/pkgconfig/",
-                    "/usr/share/pkgconfig"].join(SEPARATOR)
+    arch_depended_path = Dir.glob("/usr/lib/*/pkgconfig")
+    default_paths = [
+      "/usr/local/lib64/pkgconfig",
+      "/usr/local/libx32/pkgconfig",
+      "/usr/local/lib/pkgconfig",
+      "/usr/local/libdata/pkgconfig",
+      "/usr/local/share/pkgconfig",
+      "/opt/local/lib/pkgconfig",
+      *arch_depended_path,
+      "/usr/lib64/pkgconfig",
+      "/usr/libx32/pkgconfig",
+      "/usr/lib/pkgconfig",
+      "/usr/libdata/pkgconfig",
+      "/usr/X11R6/lib/pkgconfig",
+      "/usr/X11R6/share/pkgconfig",
+      "/usr/X11/lib/pkgconfig",
+      "/opt/X11/lib/pkgconfig",
+      "/usr/share/pkgconfig",
+    ]
+    case RUBY_PLATFORM
+    when "x86-mingw32"
+      default_paths.concat(Dir.glob("c:/msys*/mingw32/lib/pkgconfig"))
+    when "x64-mingw32"
+      default_paths.concat(Dir.glob("c:/msys*/mingw64/lib/pkgconfig"))
+    end
     libdir = ENV["PKG_CONFIG_LIBDIR"]
-    default_path = [libdir, default_path].join(SEPARATOR) if libdir
+    default_paths.unshift(libdir) if libdir
 
     pkg_config = self.class.native_pkg_config
-    return default_path unless pkg_config.absolute?
+    return default_paths.join(SEPARATOR) unless pkg_config.absolute?
+
     pkg_config_prefix = pkg_config.parent.parent
-    pkg_config_arch_depended_path =
-      Dir.glob((pkg_config_prefix + "lib/*/pkgconfig").to_s).join(SEPARATOR)
-    [pkg_config_arch_depended_path,
-     (pkg_config_prefix + "lib64/pkgconfig").to_s,
-     (pkg_config_prefix + "lib/pkgconfig").to_s,
-     (pkg_config_prefix + "libdata/pkgconfig").to_s,
-     default_path].join(SEPARATOR)
+    pkg_config_arch_depended_paths =
+      Dir.glob((pkg_config_prefix + "lib/*/pkgconfig").to_s)
+    paths = []
+    paths.concat(pkg_config_arch_depended_paths)
+    paths << (pkg_config_prefix + "lib64/pkgconfig").to_s
+    paths << (pkg_config_prefix + "libx32/pkgconfig").to_s
+    paths << (pkg_config_prefix + "lib/pkgconfig").to_s
+    paths << (pkg_config_prefix + "libdata/pkgconfig").to_s
+    if /-darwin\d[\d\.]*\z/ =~ RUBY_PLATFORM and
+        /\A(\d+\.\d+)/ =~ `sw_vers -productVersion`
+      mac_os_version = $1
+      homebrew_repository_candidates = []
+      brew_path = pkg_config_prefix + "bin" + "brew"
+      if brew_path.exist?
+        escaped_brew_path = Shellwords.escape(brew_path.to_s)
+        homebrew_repository = `#{escaped_brew_path} --repository`.chomp
+        homebrew_repository_candidates << Pathname.new(homebrew_repository)
+      else
+        homebrew_repository_candidates << pkg_config_prefix + "Homebrew"
+        homebrew_repository_candidates << pkg_config_prefix
+      end
+      homebrew_repository_candidates.each do |candidate|
+        path = candidate + "Library/Homebrew/os/mac/pkgconfig/#{mac_os_version}"
+        paths << path.to_s if path.exist?
+      end
+    end
+    paths.concat(default_paths)
+    paths.join(SEPARATOR)
   end
 
   def required_packages
@@ -302,6 +404,12 @@ class PackageConfig
     end.uniq
   end
 
+  def private_required_packages
+    requires_private.reject do |package|
+      @name == package
+    end.uniq
+  end
+
   def all_required_packages
     (requires_private + requires.reverse).reject do |package|
       @name == package
@@ -316,8 +424,6 @@ class PackageConfig
 end
 
 module PKGConfig
-  VERSION = "1.1.2"
-
   @@paths = []
   @@override_variables = {}
 
@@ -331,7 +437,7 @@ module PKGConfig
   end
 
   def msvc?
-    /mswin32/.match(RUBY_PLATFORM) and /^cl\b/.match(Config::CONFIG['CC'])
+    /mswin/.match(RUBY_PLATFORM) and /^cl\b/.match(RbConfig::CONFIG['CC'])
   end
 
   def package_config(package)
@@ -365,6 +471,10 @@ module PKGConfig
     package_config(pkg).cflags_only_I
   end
 
+  def cflags_only_other(pkg)
+    package_config(pkg).cflags_only_other
+  end
+
   def modversion(pkg)
     package_config(pkg).version
   end
@@ -404,12 +514,14 @@ module PKGConfig
                   Shellwords.shellwords(libraries))
       dldflags = dldflags.map {|s| /\s/ =~ s ? "\"#{s}\"" : s }.join(' ')
       $libs   += ' ' + libraries
-      if /mswin32/ =~ RUBY_PLATFORM
+      if /mswin/ =~ RUBY_PLATFORM
         $DLDFLAGS += ' ' + dldflags
       else
         $LDFLAGS += ' ' + dldflags
       end
-      $CFLAGS += ' ' + cflags(pkg)
+      $CFLAGS += ' ' + cflags_only_other(pkg)
+      $CXXFLAGS += ' ' + cflags_only_other(pkg)
+      $INCFLAGS += ' ' + cflags_only_I(pkg)
     end
     enough_version
   end
